@@ -1,6 +1,4 @@
 package URI::NamespaceMap;
-use strict;
-use warnings;
 use Moo 1.006000;
 use Module::Load::Conditional qw[can_load];
 use URI::Namespace;
@@ -10,6 +8,7 @@ use Sub::Quote qw( quote_sub );
 use Try::Tiny;
 use Types::Standard qw(HashRef);
 use Types::Namespace 0.004 qw(Namespace);
+use URI::NamespaceMap::ReservedLocalParts;
 use namespace::autoclean;
 
 =head1 NAME
@@ -18,11 +17,11 @@ URI::NamespaceMap - Class holding a collection of namespaces
 
 =head1 VERSION
 
-Version 0.30
+Version 1.00
 
 =cut
 
-our $VERSION = '0.30';
+our $VERSION = '1.00';
 
 
 =head1 SYNOPSIS
@@ -91,17 +90,25 @@ Returns an array of prefixes.
 
 =cut
 
-
 around BUILDARGS => sub {
 	my ($next, $self, @parameters) = @_;
 	if (ref($parameters[0]) eq 'ARRAY') {
 		return { namespace_map => $self->_guess(@{$parameters[0]}) };
 	}
-	return $self->$next(@parameters) if (@parameters > 1);
-	return $self->$next(@parameters) if (exists $parameters[0]->{namespace_map});
-	return { namespace_map => $parameters[0] };
+	if (@parameters == 1 && (! exists $parameters[0]->{namespace_map})) {
+		return { namespace_map => $parameters[0] } 
+	} else { $self->$next(@parameters) }
 };
 
+sub BUILD {
+    my ($self, $args) = @_;
+    my $r = URI::NamespaceMap::ReservedLocalParts->new(disallowed => [qw/uri/]);
+    for my $local_part (keys %{$args->{namespace_map}}) {
+        Carp::croak("$_[1] prohibited as local part")
+            if $r->is_reserved($local_part);
+    }
+}
+
 has namespace_map => (
                       is => "ro",
                       isa => HashRef[Namespace],
@@ -109,12 +116,34 @@ has namespace_map => (
                       default => quote_sub q { {} },
                      );
 
-sub add_mapping     { $_[0]->namespace_map->{$_[1]} = Namespace->assert_coerce($_[2]) }
+sub add_mapping {
+	my $r = URI::NamespaceMap::ReservedLocalParts->new(disallowed => [qw/uri/]);
+	Carp::croak("$_[1] prohibited as local part") if $r->is_reserved($_[1]);
+
+	$_[0]->namespace_map->{$_[1]} = Namespace->assert_coerce($_[2])
+}
 sub remove_mapping  { delete $_[0]->namespace_map->{$_[1]} }
 sub namespace_uri   { $_[0]->namespace_map->{$_[1]} }
 sub list_namespaces { values %{ $_[0]->namespace_map } }
 sub list_prefixes   { keys   %{ $_[0]->namespace_map } }
 
+
+=item C<< guess_and_add ( @string_or_uri ) >>
+
+Like in the constructor, an array of strings can be given, and the
+module will attempt to guess appropriate mappings, and add them to the
+map.
+
+=cut
+
+sub guess_and_add {
+	my ($self, @data) = @_;
+	my $guessed = $self->_guess(@data);
+	while (my ($name, $uri) = each %{$guessed}) {
+		$self->add_mapping($name => $uri);
+	}
+}
+
 =item C<< uri ( $prefixed_name ) >>
 
 Returns a URI for an abbreviated string such as 'foaf:Person'.
@@ -147,7 +176,6 @@ called in list context) for the given URI.
 
 =cut
 
-
 # turn the URI back into a string to mitigate unexpected behaviour
 sub _scrub_uri {
 	my $uri = shift;
@@ -178,36 +206,36 @@ sub _scrub_uri {
 			            "an unblessed %s reference", ref $uri);
 		}
 	}
-	
+
 	return $uri;
 }
 
 sub prefix_for {
 	my ($self, $uri) = @_;
-	
+
 	$uri = _scrub_uri($uri);
-	
+
 	my @candidates;
 	for my $k ($self->list_prefixes) {
 		my $v = $self->namespace_uri($k);
-		
+
 		my $nsuri = $v->as_string;
-		
+
 		# the input should always be longer than the namespace
 		next if length $nsuri > length $uri;
-		
+
 		# candidate namespace must match exactly
 		my $cns = substr($uri, 0, length $nsuri);
 		push @candidates, $k if $cns eq $nsuri;
 	}
-	
+
 	# make sure this behaves correctly when empty
 	return unless @candidates;
-	
+
 	# if this returns more than one prefix, take the
 	# shortest/lexically lowest one.
 	@candidates = sort @candidates;
-	
+
 	return wantarray ? @candidates : $candidates[0];
 }
 
@@ -224,16 +252,16 @@ may be useful for certain serialization tasks.
 
 sub abbreviate {
 	my ($self, $uri) = @_;
-	
+
 	$uri = _scrub_uri($uri);
-	
+
 	my $prefix = $self->prefix_for($uri);
-	
+
 	# XXX is this actually the most desirable behaviour?
 	return unless defined $prefix;
-	
+
 	my $nsuri = _scrub_uri($self->namespace_uri($prefix));
-	
+
 	return sprintf('%s:%s', $prefix, substr($uri, length $nsuri));
 }
 
@@ -253,34 +281,42 @@ sub _guess {
 	my $xmlns = can_load( modules => { 'XML::CommonNS' => 0 } );
 	my $rdfns = can_load( modules => { 'RDF::NS' => 20130802 } );
 	my $rdfpr = can_load( modules => { 'RDF::Prefixes' => 0 } );
-	
+
 	confess 'To resolve an array, you need at least one of RDF::NS::Curated, XML::CommonNS, RDF::NS or RDF::Prefixes' unless ($rnscu || $xmlns || $rdfns || $rdfpr);
 	my %namespaces;
-	
+	my $r = URI::NamespaceMap::ReservedLocalParts->new(disallowed => [qw/uri/]);
+
 	foreach my $entry (@data) {
-		
 		if ($entry =~ m/^[a-z]\w+$/i) {
 			# This is a prefix
-			warn "Cannot resolve '$entry' without RDF::NS::Curated, XML::CommonNS, RDF::NS" unless ($rnscu || $xmlns || $rdfns);
+			carp "Cannot resolve '$entry' without RDF::NS::Curated, XML::CommonNS, RDF::NS" unless ($rnscu || $xmlns || $rdfns);
+			my $i = 1;
+			my $prefix = $entry;
+			while ($r->is_reserved($prefix)) {
+				$prefix .= 'x';
+				carp "Cannot resolve '$entry' as tried prefix '$prefix' conflicts with method names." if ($i > 5);
+				$i++;
+			}
+
 			if ($rnscu) {
 				my $ns = RDF::NS::Curated->new;
-				$namespaces{$entry} = $ns->uri($entry);
+				$namespaces{$prefix} = $ns->uri($entry);
 			}
-			if ((! $namespaces{$entry}) && $xmlns) {
+			if ((! $namespaces{$prefix}) && $xmlns) {
 				require XML::CommonNS;
 				XML::CommonNS->import(':all');
 				try {
-					$namespaces{$entry} = XML::CommonNS->uri(uc($entry))->toString;
+					$namespaces{$prefix} = XML::CommonNS->uri(uc($entry))->toString;
 				}; # Then, XML::CommonNS doesn't have the prefix, which is OK, we just continue
 			}
-			if ((! $namespaces{$entry}) && $rdfns) {
+			if ((! $namespaces{$prefix}) && $rdfns) {
 				my $ns = RDF::NS->new;
-				$namespaces{$entry} = $ns->SELECT($entry);
+				$namespaces{$prefix} = $ns->SELECT($entry);
 			}
-			warn "Cannot resolve '$entry'" unless $namespaces{$entry};
+			carp "Cannot resolve '$entry'" unless $namespaces{$prefix};
 		} else {
 			# Lets assume a URI string
-			warn "Cannot resolve '$entry' without RDF::NS::Curated, RDF::NS or RDF::Prefixes" unless ($rnscu || $rdfns || $rdfpr);
+			carp "Cannot resolve '$entry' without RDF::NS::Curated, RDF::NS or RDF::Prefixes" unless ($rnscu || $rdfns || $rdfpr);
 			my $prefix;
 			if ($rnscu) {
 				my $ns = RDF::NS::Curated->new;
@@ -295,8 +331,14 @@ sub _guess {
 				$prefix = $context->get_prefix($entry);
 			}
 			unless ($prefix) {
-				warn "Cannot resolve '$entry'";
+				carp "Cannot resolve '$entry'";
 			} else {
+				my $i = 1;
+				while ($r->is_reserved($prefix)) {
+					$prefix .= 'x';
+					carp "Cannot resolve '$entry' as tried prefix '$prefix' conflicts with method names." if ($i > 5);
+					$i++;
+				}
 				$namespaces{$prefix} = $entry;
 			}
 		}
@@ -304,7 +346,6 @@ sub _guess {
 	return \%namespaces;
 }
 
-
 =back
 
 =head1 WARNING
@@ -314,6 +355,8 @@ prefix, because these names are defined as method for every Perl
 object by default. The method names 'new' and 'uri' are also
 forbidden. Names of methods of L<Moose::Object> must also be avoided.
 
+Using them will result in an error.
+
 =head1 AUTHORS
 
 Chris Prather, C<< <chris@prather.org> >>
@@ -324,6 +367,7 @@ Toby Inkster, C<< <tobyink@cpan.org> >>
 =head1 CONTRIBUTORS
 
 Dorian Taylor
+Paul Williams
 
 =head1 BUGS
 
@@ -338,7 +382,7 @@ You can find documentation for this module with the perldoc command.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2012,2013,2014,2015,2016 Gregory Todd Williams, Chris Prather and Kjetil Kjernsmo
+Copyright 2012,2013,2014,2015,2016,2017 Gregory Todd Williams, Chris Prather and Kjetil Kjernsmo
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
@@ -348,4 +392,3 @@ under the same terms as Perl itself.
 
 1;
 __END__
-
